#!/usr/bin/env python
# encoding: utf-8
import datetime
from calendar import monthrange
from decimal import Decimal
import logging
from pyramid.httpexceptions import HTTPUnauthorized
from ringo.lib.helpers.appinfo import get_app_location
from ringo.views.helpers import has_role
import os.path
from ringo_target.model import Target


log = logging.getLogger(__name__)
ERROR_LOG = []


def get_target(request, year):
    if request:
        target_as_str = request.db.query(Target).filter(Target.year==year).first()
        if target_as_str is not None:
            return Decimal(target_as_str.target)
        else:
            global ERROR_LOG
            if not year in ERROR_LOG:
                # We want this error only one time (per request).
                log.error("Could not find year %s in the db table targets"
                          % year)
                ERROR_LOG.append(year)
            # In this case target is canonical 0
            return Decimal('0')
    # for tests
    return Decimal(1200)


def get_new_entries(request, year, month):
    if request:
        pass
    # for tests
    return Decimal(80)


def get_cumulative_new_entries(request,  get_new_entries, year, to_month):
    return sum([get_new_entries(request, year, month) for month 
                in range(1, to_month+1)])


def get_target_per_month(target, month):
    return target / Decimal(12)


def get_cumulative_target_per_month(target, to_month):
    return sum([get_target_per_month(target, month) for month
                in range(1, to_month + 1)])


def calculate_quota(target, entries):
    if target <= Decimal(0):
        # if no target, then you sucide.
        return Decimal(100)
    return (entries * Decimal(100)) / target


def get_quota(request, get_new_entries, year, month):
    target = get_target(request, year)
    target_per_month = get_target_per_month(target, month)
    entries = get_new_entries(request, year, month)
    quota = calculate_quota(target_per_month, entries)
    color = get_color(quota, year, month)
    return (target_per_month.quantize(Decimal('1')), entries,
            quota.quantize(Decimal('1')), color)


def get_cumulative_quota(request, get_new_entries, year, to_month):
    target = get_target(request, year)
    target_per_to_month = get_cumulative_target_per_month(target, to_month)
    entries = get_cumulative_new_entries(request, get_new_entries, year,
                                         to_month)
    quota = calculate_quota(target_per_to_month, entries)
    color = get_color(quota, year, to_month)
    return (target_per_to_month.quantize(Decimal('1')), entries,
            quota.quantize(Decimal('1')), color)


def get_total_quota(request, get_new_entries, year, to_month):
    total_entries = 0
    total_targets = 0

    for year2 in range(get_startyear(request), year):
        year_quota_tuple = get_cumulative_quota(request, get_new_entries, year2,
                                                12)
        total_entries += year_quota_tuple[1]
        total_targets += year_quota_tuple[0]

    current = get_cumulative_quota(request, get_new_entries, year, to_month)
    total_entries += current[1]
    total_targets += current[0]
    
    quota = calculate_quota(total_targets, total_entries).quantize(Decimal('1'))
    color = get_color(quota, year, to_month)
    return (total_targets, total_entries, quota, color)


def get_color(quota, year, month):
    today = datetime.date.today()
    if today.year < year or today.year == year and today.month < month:
        return ""
    if quota >= Decimal(100):
        return "success"
    elif quota >= Decimal(50):
        return "warning"
    else:
        return "danger"


def get_detectsystem_data(request, get_new_entries):
    res = {}
    today = datetime.date.today()
    res['year_today'] = today.year
    res['month_today'] = get_month_name(today.month)
    res['quota_today'] = get_quota(request, get_new_entries, today.year,
                                   today.month)
    res['cumulative_quota_today'] = get_cumulative_quota(request,
        get_new_entries, today.year, today.month)
    res['total_quota_today'] = get_total_quota(request, get_new_entries,
        today.year, today.month)

    if today.month == 1:
        year_before = today.year -1
        month_before = 12
    else:
        year_before = today.year
        month_before = today.month - 1
    res['year_before'] = year_before
    res['month_before'] = get_month_name(month_before)
    res['quota_before'] = get_quota(request, get_new_entries, year_before,
                                    month_before)
    res['cumulative_quota_before'] = get_cumulative_quota(request,
        get_new_entries, year_before, month_before)
    res['total_quota_before'] = get_total_quota(request, get_new_entries,
        year_before, month_before)

    return res


def get_month_name(monthnr):
    d = {1: u"Januar", 2: u"Februar", 3: u"März", 4: u"April",
         5: u"Mai", 6: u"Juni", 7: u"Juli", 8: u"August",
         9: u"September", 10: u"Oktober", 11: u"November", 12: u"Dezember"}
    return d.get(monthnr, "???")


def get_detailed_data_per_month(request, get_new_entries, year, month):
    return {"year": year, 
            "month": month,
            "monthname": get_month_name(month),
            "quota": get_quota(request, get_new_entries, year, month),
            "cum_quota": get_cumulative_quota(request, get_new_entries, year,
                                              month),
            "total_quota": get_total_quota(request, get_new_entries, year,
                                           month)}


def get_detectsystem_detailed_data(request, get_new_entries):
    data = []
    today = datetime.date.today()
    year = today.year
    if request.params.get('year'):
        try:
            year = int(request.params.get('year'))
        except:
            log.error("Cannot parse year %s" % request.params.get('year'))
    for month in range(1, 13):
        data.append(get_detailed_data_per_month(request, get_new_entries,
                                                year, month))
    return (data, year)


def detectsystem_detail(request, get_new_entries):
    if not has_role(request.user, "beneficiary"):
        raise HTTPUnauthorized
    return get_detectsystem_detailed_data(request, get_new_entries)


def get_startyear(request):
    year = 2015
    try:
        year = int(request.registry.settings.get("efa.target.startyear", year))
    except:
        log.error("Cannot parse config option efa.target.startyear")
    return year


def get_totaltarget(request):
    total = Decimal(0)
    try:
        total = Decimal(request.registry.settings.get("efa.target.total",
                                                  total))
    except:
        log.error("Cannot parse config option efa.target.total")
    return total


def get_quota_tuple_string(tuple1):
    res = []
    for index in range(0, 4):
        res.append(str(tuple1[index]))
    return ";".join(res)


def export(db_request, end_year, get_new_entries):
    res = ["year;month;target;entries;quota;color;cum_target;cum_entries;"
           "cum_quota;cum_color;total_target;total_entries;total_quota;"
           "total_color"]
    for year in range(get_startyear(db_request), end_year + 1):
        for month in range(1, 12 + 1):
            vlist = [get_quota(db_request, get_new_entries, year, month),
                     get_cumulative_quota(db_request, get_new_entries,
                                          year, month),
                     get_total_quota(db_request, get_new_entries,
                                     year, month)]
            res2 = [str(year), str(month)]
            for values in vlist:
                res2.append(get_quota_tuple_string(values))
            res.append(";".join(res2))
    return "\n".join(res)
